<?php
namespace Fiscal\Logic\Impostos;

use \Fiscal\Entities\Notafiscal;
use \Fiscal\Entities\Itemproimpos;
use \Fiscal\Entities\Trabitproimpos;
use \Fiscal\Logic\Impostos\Exceptions\ImpostoJaCadastrado;
use Fiscal\Logic\Impostos\Exceptions\ImpostoNaoCalculado;
use \Fiscal\Logic\Impostos\Exceptions\ItemSemImposto;
use \Geral\Entities\Tipooperac;
use \Indeterminado\Entities\Impobrigtpoper;
use Windows\WinPbi00017000\TabPbi000170000004\Exception\Exceptions;


/**
 * Verifica os impostos da nota fiscal.
 *
 * Created by JetBrains PhpStorm.
 * User: paulopereira
 * Date: 29/10/12
 * Time: 14:35
 * To change this template use File | Settings | File Templates.
 */
class ValidacaoImpostos
{

    /**
     * Verifica se a filial utiliza controle fiscal. As validacoes dos impostos somente
     * serao utilizadas caso esta validacao seja verdadeira.
     *
     * @return bool
     */
    public function utilizaControleFiscal()
    {
        return \Fiscal\Util\Parametros::getIdutctrlfis() === \Geral\Entities\Paramfilial::UTILIZA_CONTROLE_FISCAL;
    }



    /**
     * Verifica se todos os impostos por itens da tabela de TRABITPROIMPOS possuem itens associados na tabela de
     * ITEMPROIMPOS.
     *
     * @param Notafiscal $notaFiscal
     * @return bool
     * @throws ItemSemImposto
     */
    public function validaImpostos(Notafiscal $notaFiscal)
    {
        // Retorna todos os itens da Trabitproimpos com cdfilial e nrlanctonf iguais aos mesmos da NF.
        $impostoItensProdTemp = Trabitproimpos::findByCdfilimpoitAndNrlancimpoitAndNrorg (
            $notaFiscal->getFilial(), $notaFiscal->getNrlanctonf(), \Geral\Util\Ambiente::getNrorg()
        );

        if ($this->todosImpostosRelacionados($impostoItensProdTemp) === false) {
            throw new ItemSemImposto();
        }

        return true;
    }

    /**
     * Verifica se todos os impostos da tabela de TRABITEMPROIMPOS (Imposto do Item Produto (Temporária)) foram
     * exportados para a tabela de ITEMPROIMPOS (Imposto do Item Produto). Deve retornar false se ao menos um dos
     * registros de TRABITEMPROIMPOS nao possuirem item associado em ITEMPROIMPOS.
     *
     * @param array $impostoItensProdTemp
     * @return bool
     */
    protected function todosImpostosRelacionados(array $impostoItensProdTemp)
    {
        $todosImpostosRelacionados = true;

        /** @var  $trabitproimpos Trabitproimpos */
        foreach ($impostoItensProdTemp as $trabitproimpos) {
            $cdfilial    = $trabitproimpos->getCdfilimpoit();
            $nrlanctonf  = $trabitproimpos->getNrlancimpoit();
            $nrseqitprod = $trabitproimpos->getNrseqitpro();
            $cdimposto   = $trabitproimpos->getCdimpostoit();

            $todosImpostosRelacionados = (bool)Itemproimpos::findOneByCdfilialAndNrlanctonfAndNrseqitprodAndCdimposto(
                $cdfilial, $nrlanctonf, $nrseqitprod, $cdimposto
            );

            if (!$todosImpostosRelacionados) {
                return false;
            }
        }

        return $todosImpostosRelacionados;
    }


    /**
     * Valida os impostos obrigatorios: Verifica se estes ja foram aplicados a NF ou a seus produtos (dependendo de
     * qual tipo de Imposto a nota possui).
     *
     * @param Notafiscal $notaFiscal
     * @return bool
     * @throws \Fiscal\Logic\Impostos\Exceptions\ImpostoNaoCalculado
     *
     */
    public function validaImpostosObrigatorios(Notafiscal $notaFiscal)
    {
        // Verifica se utiliza controle fiscal.
        if ($this->utilizaControleFiscalGeral() || $this->filialUtilizaControleFiscal($notaFiscal)) {
            $impostosNaoCalculados = $this->verificaImpostosObrigatorios($notaFiscal);

            if (!empty($impostosNaoCalculados)) {
                $stImposto = '';
                for($i=0; $i<sizeof($impostosNaoCalculados)-1; $i++){
                    $stImposto = $stImposto.$impostosNaoCalculados[$i].', ';
                }
                $stImposto = $stImposto.$impostosNaoCalculados[$i];
                
                ImpostoNaoCalculado::impostosObrigatoriosNaoCalculados($stImposto);
            }
        }

        return true;
    }
    
    public function validaImpostoJaExistenteNFE($cdimposto){
        $jaCadastrado = \Geral\Entities\Imposto::getRepository()->getImpostoJaCadastradoNFE($cdimposto);
        if($jaCadastrado)
            ImpostoJaCadastrado::impostoJaCadastrado();
    }
    
    public function validaImpostoJaExistenteXML($cdimposto){
        $jaCadastrado = \Geral\Entities\Imposto::getRepository()->getImpostoJaCadastradoXML($cdimposto);
        if($jaCadastrado)
            ImpostoJaCadastrado::impostoJaCadastrado();
    }

    /**
     * Verifica se já existe mapa de apuração criado para o imposto.
     *
     * @param $cdimposto
     * @throws \Exception
     */
    public function validaImpostoMapa($cdimposto){
        $mapaapurac = \Fiscal\Entities\Mapaapurac::findByCdimposto($cdimposto);
        if ($mapaapurac) {
            ImpostoJaCadastrado::impostoCadastradoMapa();
        }
    }

    /**
     * Verifica se os impostos foram ou nao calculados.
     *
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @return bool
     */
    protected function verificaImpostosObrigatorios(Notafiscal $notaFiscal)
    {
        /** @var $tipooperac Tipooperac */
        /** @var $impostosObrigatorios array*/
        $tipooperac           = $notaFiscal->getEntrsaidop();
        $repositorioImpobrigtpoper = \Indeterminado\Entities\Impobrigtpoper::getRepository();
        $impostosObrigatorios = $repositorioImpobrigtpoper->getImpostosTpOperacao(
            $tipooperac->getIdentrsaidop(), $tipooperac->getCdtipooper(), \Geral\Util\Ambiente::getNrorg()
        );

        $impostosNaoCalculados = array();
        /** @var  $impostoObrigatorio Impobrigtpoper */
        foreach ($impostosObrigatorios as $impostoObrigatorio) {
            if ($impostoObrigatorio['IDPRODNOTA'] === Impobrigtpoper::IMPOSTO_POR_PRODUTO) {
                if($this->verificaImpostoPorProduto($notaFiscal, $impostoObrigatorio['CDIMPOSTO']))
                    $impostosNaoCalculados[] = $impostoObrigatorio['SGIMPOSTO'];
            } else {
                if($this->verificaImpostoPorNF($notaFiscal, $impostoObrigatorio['CDIMPOSTO']))
                    $impostosNaoCalculados[] = $impostoObrigatorio['SGIMPOSTO'];
            }
        }

        return $impostosNaoCalculados;
    }

    /**
     * Verifica se os impostos da NF foram calculados. Sera usados quando os impostos incidirem na Nf.
     *
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @param \Indeterminado\Entities\Impobrigtpoper $impostoObrigatorio
     * @return bool
     */
    protected function verificaImpostoPorNF(Notafiscal $notaFiscal, $cdimposto)
    {
        /** @var $repositorioImpostonf \Fiscal\Repositories\Impostonf */
        $repositorioImpostonf = \Fiscal\Entities\Impostonf::getRepository();
        $impostosCalculados   = $repositorioImpostonf->existeImpostoPorNF(
            $notaFiscal->getFilial()->getCdfilial(),
            $notaFiscal->getNrlanctonf(),
            $cdimposto
        );
        return $impostosCalculados;
    }

    /**
     * Verifica se os impostos dos produtos foram calculados. Sera usado quando os impostos incidirem sobre os produtos.
     *
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @param \Indeterminado\Entities\Impobrigtpoper $impostoObrigatorio
     * @return bool
     */
    protected function verificaImpostoPorProduto(Notafiscal $notaFiscal, $cdimposto)
    {
        /** @var $repositorioImpobrigtpoper \Indeterminado\Repositories\Impobrigtpoper */
        $repositorioImpobrigtpoper = \Indeterminado\Entities\Impobrigtpoper::getRepository();
        $impostosCalculados = $repositorioImpobrigtpoper->existeImpostoPorProduto (
            $notaFiscal->getFilial()->getCdfilial(),
            $notaFiscal->getNrlanctonf(),
            $cdimposto
        );

        return $impostosCalculados;
    }


    /**
     * Verifica se a filial da Nota Fiscal utiliza Controle Fiscal.
     *
     * @param Notafiscal $notaFiscal
     * @return bool
     * @throws \Exception
     */
    protected function filialUtilizaControleFiscal(Notafiscal $notaFiscal)
    {
        try {
            /** @var $paramfilial \Geral\Entities\Paramfilial */
            $paramfilial = \Geral\Entities\Paramfilial::findOneByCdfilialAndNrorg (
                $notaFiscal->getFilial()->getCdfilial(), \Geral\Util\Ambiente::getNrorg()
            );
            return $paramfilial->utilizaControleFiscal();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    protected function utilizaControleFiscalGeral(){
        try {
            /** @var $paramgeral \Geral\Entities\Paramgeral */
            $paramgeral = \Geral\Entities\Paramgeral::findOneByNrorg(\Geral\Util\Ambiente::getNrorg());
                return $paramgeral->getIdutctrlfisentr();
        } catch (\Exception $e) {
            throw $e;
        }
    }

}
